/**
 * Copyright 2010-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.protocol.ssh.impl.jsch;

import java.util.Map;

import org.ow2.util.protocol.api.ProtocolException;
import org.ow2.util.protocol.ssh.api.SshProtocolConfig;

/**
 *
 * @author Loris Bouzonnet
 */
public final class SshProtocolConfigFactory {

    public static final String PASSWORD = "password";

    public static final String PRIVATE_KEY_URL = "private-key-url";

    public static final String PASSPHRASE = "passphrase";

    public static final String KNOWN_HOSTS = "known_hosts";

    public static final String HOST = "host";

    public static final String PORT = "port";

    public static final String USER = "user";

    public static final String COMPRESSION = "compression";

    public static final String PROXY_HTTP = "proxy-http";

    public static final String PROXY_SOCKS4 = "proxy-socks4";

    public static final String PROXY_SOCKS5 = "proxy-socks5";

    public static final String X11_FORWARDING = "x11-forwarding";

    private static final String CONNECTION_TIMEOUT = "connection-timeout";

    private static final String CHANNEL_TIMEOUT = "channel-timeout";


    public static SshProtocolConfig createSshProtocolConfig(final String url, final Map<String, String> context) throws ProtocolException {

        final SshProtocolConfig protocolConfig = new SshProtocolConfig();

        final String host;
        final String user;
        final String sPort;

        if (url != null) {

            final String[] userHost = url.split("@");
            final String hostPort;
            if (userHost.length == 1) {
                if (context != null) {
                    user = context.get(USER);
                } else {
                    user = null;
                }
                hostPort = userHost[0];

            } else if (userHost.length == 2) {
                if (context != null && context.get(USER) != null
                        && !context.get(USER).equals(userHost[0])) {
                    throw new ProtocolException();
                }
                user = userHost[0];
                hostPort = userHost[1];

            } else {
                throw new ProtocolException();
            }

            final String[] hostAndPort = hostPort.split(":");
            if (context != null && context.get(HOST) != null
                    && !context.get(HOST).equals(hostAndPort[0])) {
                throw new ProtocolException();
            }
            host = hostAndPort[0];
            if (hostAndPort.length == 2) {
                if (context != null && context.get(PORT) != null
                        && !context.get(PORT).equals(hostAndPort[1])) {
                    throw new ProtocolException();
                }
                sPort = hostAndPort[1];
            } else {
                if (context == null) {
                    sPort = null;
                } else {
                    sPort = context.get(PORT);
                }
            }
        } else {
            if (context == null) {
                throw new ProtocolException("The two arguments are null");
            }
            host = context.get(HOST);
            user = context.get(USER);
            sPort = context.get(PORT);
        }

        protocolConfig.setUser(user);
        protocolConfig.setHost(host);
        if (sPort != null) {
            protocolConfig.setPort(Integer.valueOf(sPort));
        }

        if (context != null) {
        	protocolConfig.setPassword(context.get(PASSWORD));
            protocolConfig.setPrivateKeyPath(context.get(PRIVATE_KEY_URL));
            protocolConfig.setPassphrase(context.get(PASSPHRASE));
            protocolConfig.setKnownHostsPath(context.get(KNOWN_HOSTS));

            final String sLevel = context.get(COMPRESSION);
            if (sLevel != null) {
                protocolConfig.setCompressionLevel(Integer.valueOf(sLevel));
            }
            protocolConfig.setProxyHttp(context.get(PROXY_HTTP));
            protocolConfig.setProxySocks4(context.get(PROXY_SOCKS4));
            protocolConfig.setProxySocks5(context.get(PROXY_SOCKS5));
            protocolConfig.setX11Forwarding(context.get(X11_FORWARDING));

            if (context.containsKey(CONNECTION_TIMEOUT)) {
                protocolConfig.setConnectionTimeout(
                        Integer.valueOf(context.get(CONNECTION_TIMEOUT)));
            }

            if (context.containsKey(CHANNEL_TIMEOUT)) {
                protocolConfig.setChannelTimeout(
                        Integer.valueOf(context.get(CHANNEL_TIMEOUT)));
            }
        }

        return protocolConfig;
    }

}
