/**
 * Copyright 2010-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.protocol.ssh.impl.jsch;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.HashMap;

import org.ow2.util.protocol.api.IProtocolSession;
import org.ow2.util.protocol.api.ProtocolConfig;
import org.ow2.util.protocol.api.ProtocolException;
import org.ow2.util.protocol.ssh.api.SshProtocolConfig;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelExec;
import com.jcraft.jsch.JSchException;
import com.jcraft.jsch.Session;

/**
 *
 * @author Loris Bouzonnet
 */
public class SshProtocolSession implements IProtocolSession {

    private final Session session;

    private final ProtocolConfig config;

    private final PrintStream out;

    private final PrintStream err;

    public SshProtocolSession(final Session session, final ProtocolConfig config) {
        this.session = session;
        this.config = config;
        if (config.getOutputStream() == null) {
            out = System.out;
        } else if (config.getOutputStream() instanceof PrintStream) {
            out = (PrintStream) config.getOutputStream();
        } else {
            out = new PrintStream(config.getOutputStream());
        }
        if (config.getErrorStream() == null) {
            err = System.err;
        } else if (config.getErrorStream() instanceof PrintStream) {
            err = (PrintStream) config.getErrorStream();
        } else {
            err = new PrintStream(config.getErrorStream());
        }
    }

    public void close() {
        session.disconnect();
    }

    public void send(final String command) throws ProtocolException {

        final Channel channel = createJschChannel("exec", command);
        try {
            channel.setInputStream(null);

            final InputStream in = channel.getInputStream();
            channel.connect();

            final byte[] tmp = new byte[1024];
            while (true) {
                while (in.available() > 0) {
                    final int i = in.read(tmp, 0, 1024);
                    if (i < 0) {
                        break;
                    }
                    out.println(new String(tmp, 0, i));
                }
                if (channel.isClosed()) {
                    err.println("exit-status: " + channel.getExitStatus());
                    break;
                }
                try {
                    Thread.sleep(1000);
                } catch (final Exception ee) {
                }
            }
            channel.disconnect();

        } catch (final IOException e) {
            throw new ProtocolException(e);
        } catch (final JSchException e) {
            throw new ProtocolException(e);
        }
    }

    public Channel createJschChannel(final String type, final String command)
            throws ProtocolException {

        final Channel channel;
        try {
            channel = session.openChannel(type);
        } catch (final JSchException e) {
            throw new ProtocolException(e);
        }

        if (channel instanceof ChannelExec) {
            ((ChannelExec) channel).setErrStream(err);
            if (command == null) {
                throw new ProtocolException();
            }
            ((ChannelExec) channel).setCommand(command);
        }

        if (config instanceof SshProtocolConfig
                && ((SshProtocolConfig) config).getX11Forwarding() != null) {
            channel.setXForwarding(true);
        }

        return channel;
    }

    protected Session getSession() {
        return session;
    }

    protected ProtocolConfig getConfig() {
        return config;
    }

    public static void main(final String[] arg) throws ProtocolException {

        final HashMap<String, String> context = new HashMap<String, String>();
        context.put(SshProtocolConfigFactory.HOST, arg[0]);
        context.put(SshProtocolConfigFactory.USER, arg[1]);

        // TODO: Windows and its heterogeneous implementations of SSH (putty, cygwin...) that use some different locations
        final String sshHome =
            System.getProperty("user.home") + File.separator + ".ssh" + File.separator;
        if (new File(sshHome).exists()) {
            context.put(SshProtocolConfigFactory.PRIVATE_KEY_URL,
                    sshHome + "id_rsa");
            context.put(SshProtocolConfigFactory.KNOWN_HOSTS,
                    sshHome + "known_hosts");
        } else {
            context.put(SshProtocolConfigFactory.PASSWORD, arg[3]);
        }

        final SshProtocolConfig config = SshProtocolConfigFactory
                .createSshProtocolConfig(null, context);
        config.setErrorStream(System.err);
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        config.setOutputStream(outputStream);
        final SshProtocolSession sshSession = new SshProtocol()
                .createSession(config);
        sshSession.send(arg[2]);
        sshSession.close();
        System.out.print(outputStream.toString().trim());
    }
}
