/**
 * Copyright 2010-2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.protocol.ssh.impl.jsch;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;

import org.ow2.util.file.FileUtils;
import org.ow2.util.protocol.api.IShell;
import org.ow2.util.protocol.api.IShellFormatter;
import org.ow2.util.protocol.api.ProtocolException;
import org.ow2.util.protocol.impl.shell.ShShellFormatter;
import org.ow2.util.protocol.ssh.api.SshProtocolConfig;

import com.jcraft.jsch.Channel;
import com.jcraft.jsch.JSchException;

/**
 *
 * @author Loris Bouzonnet
 */
public class SshShell implements IShell {

	private final SshProtocolSession protocolSession;

	private Channel channel;

	private PrintWriter printWriter;

	private final IShellFormatter formatter;

	public SshShell(final SshProtocolSession protocolSession, final IShellFormatter formatter) throws ProtocolException {
		this.protocolSession = protocolSession;
		this.formatter = formatter;
		init();
	}

	private void init()  throws ProtocolException {

		channel =
			protocolSession.createJschChannel("shell", null);
		channel.setInputStream(protocolSession.getConfig().getInputStream());
		channel.setOutputStream(protocolSession.getConfig().getOutputStream());
		try {
			channel.connect();
		} catch (final JSchException e) {
			throw new ProtocolException(e);
		}
		try {
			printWriter = new PrintWriter(channel.getOutputStream());
		} catch (final IOException e) {
			throw new ProtocolException(e);
		}
	}

	public void close() {
		FileUtils.close(printWriter);
		channel.disconnect();
	}

	public void execute(final String command) throws ProtocolException {
		printWriter.println(command);
		printWriter.flush();
	}

	public void setVariable(final String name, final String value) throws ProtocolException {
		execute(formatter.formatSet(name, value));
	}

	public void unsetVariable(final String name) throws ProtocolException {
		execute(formatter.formatUnset(name));
	}

	public static void main(final String[] arg) throws ProtocolException {


		final HashMap<String, String> context = new HashMap<String, String>();
		context.put(SshProtocolConfigFactory.PRIVATE_KEY_URL,
				System.getProperty("user.home") + File.separator + ".ssh" + File.separator + "id_rsa");
		context.put(SshProtocolConfigFactory.KNOWN_HOSTS,
				System.getProperty("user.home") + File.separator + ".ssh" + File.separator + "known_hosts");
		context.put(SshProtocolConfigFactory.HOST, arg[0]);
		context.put(SshProtocolConfigFactory.USER, arg[1]);

		final SshProtocolConfig config =
			SshProtocolConfigFactory.createSshProtocolConfig(null, context);
		config.setInputStream(System.in);
		config.setOutputStream(System.out);

		final SshProtocolSession sshSession =
			new SshProtocol().createSession(config);
		final IShellFormatter formatter = new ShShellFormatter();
		final SshShell shell = new SshShell(sshSession, formatter);
		shell.setVariable("welcome", "\"Hello " + System.getProperty("user.name") + "\"");
		shell.execute("echo " + formatter.formatVariable("welcome"));
	}

}
